@@ -18,7 +18,7 @@ setting `WARN_ACCURACY` to false.
 import numpy as _np
 from warnings import warn as _warn
 
-from . import basis as _basis
+from . import sve as _sve
 from . import poly as _poly
 from . import kernel as _kernel
 
@@ -39,14 +39,14 @@ def load(statistics, Lambda, h5file=None):
 
     kernel_type = {"F": _kernel.LogisticKernel,
                    "B": _kernel.RegularizedBoseKernel}[statistics]
-    basis = _basis.DimensionlessBasis(statistics, float(Lambda),
-                                      kernel=kernel_type(Lambda))
-    return Basis(statistics, Lambda, (basis.u, basis.s, basis.v))
+    kernel = kernel_type(float(Lambda))
+    sve_result = _sve.compute(kernel)
+    return Basis(statistics, Lambda, sve_result)
 
 
 class Basis:
     def __init__(self, statistics, Lambda, sve_result):
-        u, s, v = sve_result
+        u, s, v = sve_result.part()
         self._statistics = statistics
         self._Lambda = Lambda
         self._u = u
@@ -54,8 +54,8 @@ class Basis:
         self._v = v
 
         conv_radius = 40 * Lambda
-        _even_odd = {'F': 'odd', 'B': 'even'}[statistics]
-        self._uhat = u.hat(_even_odd, conv_radius)
+        even_odd = {'F': 'odd', 'B': 'even'}[statistics]
+        self._uhat = _poly.PiecewiseLegendreFT(u, even_odd, conv_radius)
 
     @property
     def Lambda(self):
@@ -94,9 +94,8 @@ class Basis:
     def compute_unl(self, n, whichl=None):
         """Compute transformation matrix from IR to Matsubara frequencies"""
         n = _np.ravel(n)
-        freq = {'F': 'odd', 'B': 'even'}[self._statistics]
-        uhat = _select(self._u, whichl).hat(freq)
-        return _np.squeeze(uhat(2 * n + uhat.zeta).T)
+        nn = 2 * n + self._uhat.zeta
+        return _np.squeeze(_select(self._uhat, whichl)(nn).T)
 
     def num_sections_x(self):
         "Number of sections of piecewise polynomial representation of u_l(x)"
