@@ -142,10 +142,6 @@ class PiecewiseLegendrePoly:
         ddata *= scale.reshape(_scale_shape)
         return self.__class__(ddata, self, symm=(-1)**n * self.symm)
 
-    def hat(self, freq, n_asymp=None):
-        """Get Fourier transformed object"""
-        return PiecewiseLegendreFT(self, freq, n_asymp)
-
     def roots(self, alpha=2):
         """Find all roots of the piecewise polynomial
 
@@ -189,14 +185,9 @@ class PiecewiseLegendrePoly:
     @property
     def ndim(self): return self.data.ndim - 2
 
-    def _in_domain(self, x):
-        return (x >= self.xmin).all() and (x <= self.xmax).all()
-
     def _split(self, x):
         """Split segment"""
-        if not self._in_domain(x):
-            raise ValueError("x must be in [%g, %g]" % (self.xmin, self.xmax))
-
+        x = _util.check_range(x, self.xmin, self.xmax)
         i = self.knots.searchsorted(x, 'right').clip(None, self.nsegments)
         i -= 1
         xtilde = x - self._xm[i]
@@ -248,30 +239,23 @@ class PiecewiseLegendreFT:
         model = self._model if self._model is None else self._model[l]
         return self.__class__(self.poly[l], self.freq, self.n_asymp, model)
 
+    @_util.ravel_argument(last_dim=True)
     def __call__(self, n):
         """Obtain Fourier transform of polynomial for given frequencies"""
-        # Evaluate only on unique frequencies
-        n_unique, n_where = np.unique(n.ravel(), return_inverse=True)
-        result_flat = self._call_impl(n_unique)[..., n_where]
-        return result_flat.reshape(self.poly.shape + n.shape)
-
-    def _call_impl(self, n):
-        """Obtain Fourier transform of polynomial for given frequencies"""
-        n = check_reduced_matsubara(n, self.zeta)
-        n_flat = n.ravel()
-        result_flat = _compute_unl_inner(self.poly, n_flat)
+        n = _util.check_reduced_matsubara(n, self.zeta)
+        result = _compute_unl_inner(self.poly, n)
 
         # We use the asymptotics at frequencies larger than conv_radius
         # since it has lower relative error.
-        cond_outer = np.abs(n_flat) >= self.n_asymp
+        cond_outer = np.abs(n) >= self.n_asymp
         if cond_outer.any():
-            n_outer = n_flat[cond_outer]
-            result_flat[..., cond_outer] = self._model.giw(n_outer).T
+            n_outer = n[cond_outer]
+            result[..., cond_outer] = self._model.giw(n_outer).T
 
-        return result_flat.reshape(self.poly.shape + n.shape)
+        return result
 
-    def extrema(self, part=None, grid=None):
-        """Obtain extrema of fourier-transformed polynomial."""
+    def extrema(self, *, part=None, grid=None, positive_only=False):
+        """Obtain extrema of Fourier-transformed polynomial."""
         if self.poly.shape:
             raise ValueError("select single polynomial")
         if grid is None:
@@ -280,7 +264,23 @@ class PiecewiseLegendreFT:
         f = self._func_for_part(part)
         x0 = _roots.discrete_extrema(f, grid)
         x0 = 2 * x0 + self.zeta
-        return _symmetrize_matsubara(x0)
+        if not positive_only:
+            x0 = _symmetrize_matsubara(x0)
+        return x0
+
+    def sign_changes(self, *, part=None, grid=None, positive_only=False):
+        """Obtain sign changes of Fourier-transformed polynomial."""
+        if self.poly.shape:
+            raise ValueError("select single polynomial")
+        if grid is None:
+            grid = self._DEFAULT_GRID
+
+        f = self._func_for_part(part)
+        x0 = _roots.find_all(f, grid, type='discrete')
+        x0 = 2 * x0 + self.zeta
+        if not positive_only:
+            x0 = _symmetrize_matsubara(x0)
+        return x0
 
     def _func_for_part(self, part=None):
         if part is None:
@@ -299,29 +299,6 @@ class PiecewiseLegendreFT:
             raise ValueError("part must be either 'real' or 'imag'")
 
 
-def check_reduced_matsubara(n, zeta=None):
-    """Checks that ``n`` is a reduced Matsubara frequency.
-
-    Check that the argument is a reduced Matsubara frequency, which is an
-    integer obtained by scaling the freqency `w[n]` as follows::
-
-        beta / np.pi * w[n] == 2 * n + zeta
-
-    Note that this means that instead of a fermionic frequency (``zeta == 1``),
-    we expect an odd integer, while for a bosonic frequency (``zeta == 0``),
-    we expect an even one.  If ``zeta`` is omitted, any one is fine.
-    """
-    n = np.asarray(n)
-    if not np.issubdtype(n.dtype, np.integer):
-        nfloat = n
-        n = nfloat.astype(int)
-        if not (n == nfloat).all():
-            raise ValueError("reduced frequency n must be integer")
-    if zeta is not None:
-        if not (n & 1 == zeta).all():
-            raise ValueError("n have wrong parity")
-    return n
-
 
 def _imag_power(n):
     """Imaginary unit raised to an integer power without numerical error"""
@@ -429,7 +406,7 @@ class _PowerModel:
     @_util.ravel_argument()
     def giw(self, wn):
         """Return model Green's function for vector of frequencies"""
-        wn = check_reduced_matsubara(wn)
+        wn = _util.check_reduced_matsubara(wn)
         result_dtype = np.result_type(1j, wn, self.moments)
         result = np.zeros((wn.size, self.nl), result_dtype)
         inv_iw = 1j * np.pi/2 * wn
@@ -476,6 +453,8 @@ def _refine_grid(knots, alpha):
 
 
 def _symmetrize_matsubara(x0):
+    if not x0.size:
+        return x0
     if not (x0[1:] >= x0[:-1]).all():
         raise ValueError("set of Matsubara points not ordered")
     if not (x0[0] >= 0):
